from enum import Enum

class Error(Exception):
    pass


class Runner(Enum):
    SGX             = 1
    NATIVE          = 2

    @staticmethod
    def from_str(str):
        lower_str = str.lower()

        if lower_str == "sgx":
            return Runner.SGX
        if lower_str == "native":
            return Runner.NATIVE

        raise Error("No matching runner for {}".format(str))


    def to_str(self):
        if self == Runner.SGX:
            return "runner_sgx"
        if self == Runner.NATIVE:
            return "runner_native"


    def has_hardcoded_key(self):
        if self == Runner.NATIVE:
            return True

        return False



import os
from .runner import Runner

DEFAULT_LOG_LEVEL = "info"
STUBS_FOLDER = os.path.join(os.path.dirname(os.path.abspath(__file__)), "stubs")


# Actual crates/modules
RUST_LAZY = "#[macro_use] extern crate lazy_static;\n" # to be checked before adding, because it could be already present
RUST_INSERT_INPUT = "\t\tm.insert({id}, crate::{name} as fn(&[u8]));\n"
RUST_INSERT_ENTRY = "\t\tm.insert({id}, crate::{name} as fn(&[u8]) -> ResultMessage);\n"
RUST_INSERT_HANDLER = "\t\tm.insert({id}, crate::{name} as fn(&[u8]) -> Vec<u8>);\n"


# Stubs
STUB_MODS_USES = "mods_uses.rs"
STUB_OUTPUT = "output.rs"
STUB_REQUEST = "request.rs"
STUB_CONSTANTS = "constants.rs"
STUB_MAIN = "main.rs"
STUB_AUTH_EXEC = "__authentic_execution.rs"
CARGO_DEPENDENCIES = "common_deps.toml"

DEFAULT_RUNNER = Runner.SGX
STUB_RUNNER_RUN = "__run.rs"
STUB_RUNNER_DEPS = "dependencies.toml"

KEY_LENGTH = 16


# Starting entrypoint index
# 0 is set_key, 1 is attest, 2 is handle_input, 3 is handle_handler
START_ENTRY_INDEX   = 4
# Starting indexes of inputs, outputs, requests and handlers
# They need to have different indexes, because the `index` field in Connection does
# not distinguish between them. If the same index is used for different types, bad
# things can happen. Moreover, having these "ranges" allow us do identify what is
# an index: e.g., 25848 is an output, 44 is an input, etc.
# We believe 16384 values for each type is more than enough for a single module.
START_INPUT_INDEX   = 0
START_OUTPUT_INDEX  = 16384
START_REQUEST_INDEX = 32768
START_HANDLER_INDEX = 49152


# Regex
# Note: [ \t] means space or tab. I use this when I don't explicitly allow newlines (e.g., in comments)
#       \s means any space char (also newlines). This is for non-comment lines
REGEX_LAZY = "^\s*#\s*[\s*macro_use\s*]\s*extern\s*crate\s*lazy_static\s*;"
REGEX_OUTPUT = "^[ \t]*//@[ \t]*sm_output[ \t]*\([ \t]*(?P<fname>_?[a-zA-Z]+[_a-zA-Z0-9]*)[ \t]*\)[ \t]*$"
REGEX_INPUT = "^[ \t]*//@[ \t]*sm_input[ \t]*\n\s*pub\s+fn\s+(?P<fname>_?[a-zA-Z]+[_a-zA-Z0-9]*)\s*\(\s*_?[a-zA-Z]+[_a-zA-Z0-9]*\s*:\s*&\s*\[\s*u8\s*]\s*\)\s*\{"
REGEX_ENTRY = "^[ \t]*//@[ \t]*sm_entry[ \t]*\n\s*pub\s+fn\s+(?P<fname>_?[a-zA-Z]+[_a-zA-Z0-9]*)\s*\(\s*_?[a-zA-Z]+[_a-zA-Z0-9]*\s*:\s*&\s*\[\s*u8\s*]\s*\)\s*->\s*ResultMessage\s*\{"
REGEX_REQUEST = "^[ \t]*//@[ \t]*sm_request[ \t]*\([ \t]*(?P<fname>_?[a-zA-Z]+[_a-zA-Z0-9]*)[ \t]*\)[ \t]*$"
REGEX_HANDLER = "^[ \t]*//@[ \t]*sm_handler[ \t]*\n\s*pub\s+fn\s+(?P<fname>_?[a-zA-Z]+[_a-zA-Z0-9]*)\s*\(\s*_?[a-zA-Z]+[_a-zA-Z0-9]*\s*:\s*&\s*\[\s*u8\s*]\s*\)\s*->\s*Vec\s*<\s*u8\s*>\s*\{"



import sys
import argparse
import logging
import colorlog
import os
import base64

from . import conf
from .runner import Runner

def _set_parser():
    parser = argparse.ArgumentParser(description='Rust code generator for Authentic Execution')
    parser.add_argument('-l', '--loglevel', nargs='?', default=conf.DEFAULT_LOG_LEVEL, type=__log_level)
    parser.add_argument('-i', '--input', required=True, type=__input_dir, help='Input folder of the software module')
    parser.add_argument('-o', '--output', required=True, type=__output_dir, help='Output folder of the software module')
    parser.add_argument('-m', '--moduleid', required=True, type=__int16bits, help='16-bit Module ID')
    parser.add_argument('-e', '--emport', required=True, type=__int16bits, help='EM TCP port')
    parser.add_argument('-r', '--runner', type=__runner, required=False, default=conf.DEFAULT_RUNNER, help='Runner name')
    parser.add_argument('-s', '--spkey', required=False, type=__sp_key, help='Path to ra_sp public key')
    parser.add_argument('-p', '--print', help='Output JSON file (module infos)')
    return parser


def _set_logging(loglevel):
    log = logging.getLogger()

    format_str = '%(asctime)s.%(msecs)03d - %(levelname)-8s: %(message)s'
    date_format = '%H:%M:%S' #'%Y-%m-%d %H:%M:%S'
    if os.isatty(2):
        cformat = '%(log_color)s' + format_str
        colors = {'DEBUG': 'reset',
                  'INFO': 'bold_green',
                  'WARNING': 'bold_yellow',
                  'ERROR': 'bold_red',
                  'CRITICAL': 'bold_red'}
        formatter = colorlog.ColoredFormatter(cformat, date_format,
                                              log_colors=colors)
    else:
        formatter = logging.Formatter(format_str, date_format)

    log.setLevel(loglevel)
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(formatter)
    log.addHandler(stream_handler)


def __int16bits(arg):
    arg = int(arg)
    if arg < 0 or arg > 65535:
        raise argparse.ArgumentTypeError("Invalid Module ID: must be between 0 and 65535")

    return arg


def __str16bytes(arg):
    if len(arg) > 16:
        raise argparse.ArgumentTypeError("Invalid key: must be a 16-bytes string")

    remaining = 16 - len(arg)

    return "_" * remaining + arg


def __runner(arg):
    try:
        return Runner.from_str(arg)
    except:
        raise argparse.ArgumentTypeError("Runner does not exist")


def __log_level(arg):
    arg = arg.lower()

    if arg == "critical":
        return logging.CRITICAL
    if arg == "error":
        return logging.ERROR
    if arg == "warning":
        return logging.WARNING
    if arg == "info":
        return logging.INFO
    if arg == "debug":
        return logging.DEBUG
    if arg == "notset":
        return logging.NOTSET

    raise argparse.ArgumentTypeError("Invalid log level")


def __input_dir(arg):
    if os.path.isdir(arg):
        return arg
    else:
        raise argparse.ArgumentTypeError("Input dir does not exist")


def __output_dir(arg):
    if not os.path.exists(arg):
        return arg
    else:
        raise argparse.ArgumentTypeError("Output dir {} already exists".format(arg))


def __sp_key(arg):
    if os.path.exists(arg):
        return arg
    else:
        raise argparse.ArgumentTypeError("ra_sp public key file does not exist")



import logging
import os
import re
import toml
import base64

from . import conf
from .utils import _parse_annotations, _write_module_info, _prepare_output_dir, \
                                    _check_input_module, _copy_main, _add_fields, \
                                    _generate_key
from .initialization import _set_parser, _set_logging


def __run(args, cargo):
    out_src = os.path.join(args.output, "src")
    module_name = cargo["package"]["name"]

    ## lib.rs file ##
    ## In this section, we update lib.rs:
    ##      - parse the annotations (inputs, outputs, entry points)
    ##      - add imports (for authentic execution functions and constants)

    lib_file = os.path.join(out_src, "lib.rs")

    # parse annotations
    content, data = _parse_annotations(lib_file)

    # read imports
    with open(os.path.join(conf.STUBS_FOLDER, conf.STUB_MODS_USES), "r") as f:
        mods_uses = f.read()

    # search for lazy_static import (if already present, we do not add it again)
    lazy = ""
    if not re.match(conf.REGEX_LAZY, content, re.MULTILINE):
        lazy = conf.RUST_LAZY

    # write new content
    complete_lib = lazy + "\n" + mods_uses + "\n" + content
    with open(lib_file, "w") as f:
        f.write(complete_lib)

    ## Authentic Execution file ##
    ## In this section, we add to the project all the needed for authentic execution
    ## we also need to update some data structures with the information retrieved
    ## before

    with open(os.path.join(conf.STUBS_FOLDER, conf.STUB_CONSTANTS), "r") as f:
        constants = f.read()

    # add inputs entrypoints, handlers functions to hashmaps, so that they can
    # be called given their ID
    inputs = data["inputs"]
    inputs_fn = ""
    for input in inputs:
        inputs_fn += conf.RUST_INSERT_INPUT.format(id=inputs[input], name=input)

    entrypoints = data["entrypoints"]
    entrypoints_fn = ""
    for entry in entrypoints:
        entrypoints_fn += conf.RUST_INSERT_ENTRY.format(id=entrypoints[entry], name=entry)

    handlers = data["handlers"]
    handlers_fn = ""
    for handler in handlers:
        handlers_fn += conf.RUST_INSERT_HANDLER.format(id=handlers[handler], name=handler)

    # format constants with module's info
    constants = constants.format(id=args.moduleid, em_port=args.emport,
                    name=module_name, inputs=inputs_fn,
                    entrypoints=entrypoints_fn, handlers=handlers_fn)

    # add constants to authentic_execution file, add the file to project
    with open(os.path.join(conf.STUBS_FOLDER, conf.STUB_AUTH_EXEC), "r") as f:
        auth_exec = f.read()

    auth_exec = auth_exec.replace("{CONSTANTS}", constants)

    with open(os.path.join(out_src, conf.STUB_AUTH_EXEC), "w") as f:
        f.write(auth_exec)

    ## Main and other files ##
    ## Here, we will add the logic for main(): the project will not be a Cargo lib
    ## anymore, but an executable

    _copy_main(out_src, cargo)

    # add runner
    # sgx: the master key is retrieved by means of Remote Attestation (RA)
    # native: the master key is generated here and hardcoded inside the code
    runner = args.runner
    runner_folder = os.path.join(conf.STUBS_FOLDER, runner.to_str())

    # add __run.rs, that includes the main logic and the logic for the key
    with open(os.path.join(runner_folder, conf.STUB_RUNNER_RUN), "r") as f:
        runner_file = f.read()

    if runner.has_hardcoded_key():
        # key is hardcoded
        master_key = _generate_key()
        encoded_key = base64.b64encode(master_key).decode()
        runner_file = runner_file.replace("___MODULE_KEY___", encoded_key)
    else:
        # key is retrieved through remote attestation
        master_key = None
        encoded_key = None
        if args.spkey:
            with open(args.spkey, "r") as f:
                sp_key = f.read() + "\\0"
                runner_file = runner_file.replace("__SP_VKEY_PEM__", sp_key)
        else:
            logging.warning("ra_sp public key not provided as input! RA won't work")

    with open(os.path.join(out_src, conf.STUB_RUNNER_RUN), "w") as f:
        f.write(runner_file)

    ## Finally, edit Cargo.toml adding the needed dependencies ##

    # general dependencies (common to all runners)
    try:
        common_deps = toml.load(os.path.join(conf.STUBS_FOLDER, conf.CARGO_DEPENDENCIES))
        _add_fields(cargo, common_deps)
    except Exception as e:
        logging.error("Common deps file not found")
        raise e

    # runner dependencies
    try:
        runner_deps = toml.load(os.path.join(runner_folder, conf.STUB_RUNNER_DEPS))
        _add_fields(cargo, runner_deps)
    except Exception as e:
        logging.warn("Runner dependencies file not found")

    # write to cargo
    with open(os.path.join(args.output, "Cargo.toml"), "w") as f:
        toml.dump(cargo, f)

    # write module info to output file (if specified)
    if args.print:
        _write_module_info(args.print, module_name, args.moduleid, data, encoded_key)

    logging.debug("Done")

    return data, master_key


def generate(args):
    try:
        # check if the input dir is a correct Rust Cargo module
        logging.debug("Checking input project..")
        cargo = _check_input_module(args.input)

        # copy dir
        logging.debug("Creating output project..")
        _prepare_output_dir(args.input, args.output)

        # generate code
        logging.debug("Generating code..")
        return __run(args, cargo)

    except Exception as e:
        logging.error(e)
        exit(1)


def __main():
    # argument parser
    parser = _set_parser()
    args = parser.parse_args()

    # set logging params
    _set_logging(args.loglevel)
    generate(args)


if __name__ == "__main__":
    __main()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rust-sgx-gen",
    version="0.1.6.1",
    author="Gianluca Scopelliti",
    author_email="gianlu.1033@gmail.com",
    description="Rust code generator for the Authentic Execution framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/gianlu33/rust-sgx-gen",
    packages=setuptools.find_packages(),
    install_requires=['toml==0.10.2', 'colorlog==4.6.2'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': ['rust-sgx-gen = rustsgxgen.generator:__main']
    },
    include_package_data=True,
)



import os
import logging
import toml
import re
import subprocess
import json
import distutils
from distutils import dir_util

from . import conf

class Error(Exception):
    pass


def _prepare_output_dir(input, output):
    dir_util.copy_tree(input, output)


def _check_input_module(path):
    src = os.path.join(path, "src")
    main = os.path.join(src, "main.rs")
    lib = os.path.join(src, "lib.rs")
    cargo = os.path.join(path, "Cargo.toml")

    # Check if the path is a valid Cargo project
    retval = subprocess.call(["cargo",  "verify-project", "--manifest-path", cargo], stdout=open(os.devnull, 'wb'), stderr=open(os.devnull, 'wb'))

    if retval != 0:
        raise Error("The input path is not a valid Cargo project!")

    # Check absence of main.rs
    if os.path.exists(main):
        raise Error("main.rs file must not exist! Check the rules")

    # Check presence of lib.rs
    if not os.path.exists(lib):
        raise Error("lib.rs file does not exist")

    # Check Cargo.toml structure and dependencies
    cargo = toml.load(cargo)
    if "lib" in cargo:
        raise Error("The Cargo.toml file must not contain a [lib] section!")
    if "bin" in cargo:
        raise Error("The Cargo.toml file must not contain a [[bin]] section!")

    return cargo


def _parse_annotations(file):
    #logging.debug(annotation_file)

    with open(file, "r") as f:
        content = f.read()

    data = {}

    data["inputs"] = __parse_inputs(content)
    content, data["outputs"] = __parse_outputs(content)
    data["entrypoints"] = __parse_entrypoints(content)
    data["handlers"] = __parse_handlers(content)
    content, data["requests"] = __parse_requests(content)

    return content, data


def __parse_inputs(content):
    return __parse(content, conf.REGEX_INPUT, conf.START_INPUT_INDEX)


def __parse_outputs(content):
    return __parse_inject(content, conf.STUB_OUTPUT, conf.REGEX_OUTPUT, conf.START_OUTPUT_INDEX)


def __parse_entrypoints(content):
    return __parse(content, conf.REGEX_ENTRY, conf.START_ENTRY_INDEX)


def __parse_handlers(content):
    return __parse(content, conf.REGEX_HANDLER, conf.START_HANDLER_INDEX)


def __parse_requests(content):
    return __parse_inject(content, conf.STUB_REQUEST, conf.REGEX_REQUEST, conf.START_REQUEST_INDEX)


def __parse(content, regex, start_index):
    p = re.compile(regex, re.MULTILINE | re.ASCII)
    results = p.findall(content)

    return { v : i for (i, v) in enumerate(results, start_index) }


def __parse_inject(content, stub, regex, start_index):
    res_dict = {}
    i = 0
    cnt = 0

    # read stub from file
    with open(os.path.join(conf.STUBS_FOLDER, stub), "r") as f:
        fn = f.read()

    p = re.compile(regex, re.MULTILINE | re.ASCII)
    results = p.finditer(content)

    for result in results:
        id = start_index + i
        fname = result.group(1)
        end = result.end()
        #logging.debug("fname: {} end: {}".format(fname, end))
        pos = end + cnt
        inj_fn = fn.format(name=fname, id=id)

        content = content[:pos] + inj_fn + content[pos:]
        cnt += len(inj_fn)

        res_dict[fname] = id
        i += 1

    return content, res_dict


def _write_module_info(file, name, id, data, key=None):
    if key is not None:
        module_info = __helper_write_indexes([(name, "name"), (id, "id"), (key, "key")])
    else:
        module_info = __helper_write_indexes([(name, "name"), (id, "id")])

    content = {**module_info, **data}

    with open(file, 'w', encoding='utf-8') as f:
        json.dump(content, f, ensure_ascii=False, indent=4)


def __helper_write_indexes(data):
    section = {}

    for ind, value in data:
        section[value] = ind

    return section


def _copy_main(src, cargo):
    with open(os.path.join(conf.STUBS_FOLDER, conf.STUB_MAIN), "r") as f:
        content = f.read()

    content = content.format(cargo["package"]["name"])

    with open(os.path.join(src, conf.STUB_MAIN), "w") as f:
        f.write(content)


def _add_fields(dest, src):
    for section in src.keys():
        if section not in dest:
            dest[section] = {}

        for key in src[section].keys():
            if key in dest[section]:
                logging.warn("{} {} already in destination cargo file, overwriting".format(section, key))

            dest[section][key] = src[section][key]


def _generate_key(length=conf.KEY_LENGTH):
    return os.urandom(length)
